@@ -13,7 +13,6 @@ use mesa_rust::pipe::device::load_screens;
 use mesa_rust::pipe::fence::*;
 use mesa_rust::pipe::resource::*;
 use mesa_rust::pipe::screen::*;
-use mesa_rust::pipe::transfer::*;
 use mesa_rust_gen::*;
 use mesa_rust_util::math::SetBitIndices;
 use mesa_rust_util::static_assert;
@@ -39,59 +38,75 @@ pub struct Device {
     pub clc_versions: Vec<cl_name_version>,
     pub custom: bool,
     pub embedded: bool,
-    pub has_timestamp: bool, // Cached to keep API fast
     pub extension_string: String,
     pub extensions: Vec<cl_name_version>,
     pub spirv_extensions: Vec<CString>,
     pub clc_features: Vec<cl_name_version>,
     pub formats: HashMap<cl_image_format, HashMap<cl_mem_object_type, cl_mem_flags>>,
     pub lib_clc: NirShader,
+    pub caps: DeviceCaps,
     helper_ctx: Mutex<PipeContext>,
 }
 
+#[derive(Default)]
+pub struct DeviceCaps {
+    pub has_3d_image_writes: bool,
+    pub has_images: bool,
+    pub has_rw_images: bool,
+    pub has_timestamp: bool,
+    pub image_2d_size: u32,
+    pub max_read_images: u32,
+    pub max_write_images: u32,
+    pub timer_resolution: u32,
+}
+
+impl DeviceCaps {
+    fn new(screen: &PipeScreen) -> Self {
+        let cap_timestamp = screen.param(pipe_cap::PIPE_CAP_QUERY_TIMESTAMP) != 0;
+        let timer_resolution = screen.param(pipe_cap::PIPE_CAP_TIMER_RESOLUTION) as u32;
+
+        let max_write_images =
+            Self::shader_param(screen, pipe_shader_cap::PIPE_SHADER_CAP_MAX_SHADER_IMAGES) as u32;
+        let max_read_images =
+            Self::shader_param(screen, pipe_shader_cap::PIPE_SHADER_CAP_MAX_SAMPLER_VIEWS) as u32;
+        let image_2d_size = screen.param(pipe_cap::PIPE_CAP_MAX_TEXTURE_2D_SIZE) as u32;
+
+        let has_images = screen.param(pipe_cap::PIPE_CAP_TEXTURE_SAMPLER_INDEPENDENT) != 0 &&
+            screen.param(pipe_cap::PIPE_CAP_IMAGE_STORE_FORMATTED) != 0 &&
+            // The minimum value is 8 if CL_DEVICE_IMAGE_SUPPORT is CL_TRUE
+            max_read_images >= 8 &&
+            // The minimum value is 8 if CL_DEVICE_IMAGE_SUPPORT is CL_TRUE
+            max_write_images >= 8 &&
+            // The minimum value is 2048 if CL_DEVICE_IMAGE_SUPPORT is CL_TRUE
+            image_2d_size >= 2048;
+
+        Self {
+            has_images: has_images,
+            has_timestamp: cap_timestamp && timer_resolution > 0,
+            image_2d_size: has_images.then_some(image_2d_size).unwrap_or_default(),
+            max_read_images: has_images.then_some(max_read_images).unwrap_or_default(),
+            max_write_images: has_images.then_some(max_write_images).unwrap_or_default(),
+            timer_resolution: timer_resolution,
+            ..Default::default()
+        }
+    }
+
+    fn shader_param(screen: &PipeScreen, cap: pipe_shader_cap) -> i32 {
+        screen.shader_param(pipe_shader_type::PIPE_SHADER_COMPUTE, cap)
+    }
+}
+
 pub trait HelperContextWrapper {
     #[must_use]
     fn exec<F>(&self, func: F) -> PipeFence
     where
         F: Fn(&HelperContext);
 
-    fn buffer_map_directly(
-        &self,
-        res: &PipeResource,
-        offset: i32,
-        size: i32,
-        rw: RWFlags,
-    ) -> Option<PipeTransfer>;
-
-    fn buffer_map_coherent(
-        &self,
-        res: &PipeResource,
-        offset: i32,
-        size: i32,
-        rw: RWFlags,
-    ) -> Option<PipeTransfer>;
-
-    fn texture_map_directly(
-        &self,
-        res: &PipeResource,
-        bx: &pipe_box,
-        rw: RWFlags,
-    ) -> Option<PipeTransfer>;
-
-    fn texture_map_coherent(
-        &self,
-        res: &PipeResource,
-        bx: &pipe_box,
-        rw: RWFlags,
-    ) -> Option<PipeTransfer>;
-
     fn create_compute_state(&self, nir: &NirShader, static_local_mem: u32) -> *mut c_void;
     fn delete_compute_state(&self, cso: *mut c_void);
     fn compute_state_info(&self, state: *mut c_void) -> pipe_compute_state_object_info;
     fn compute_state_subgroup_size(&self, state: *mut c_void, block: &[u32; 3]) -> u32;
 
-    fn unmap(&self, tx: PipeTransfer);
-
     fn is_create_fence_fd_supported(&self) -> bool;
     fn import_fence(&self, fence_fd: &FenceFd) -> PipeFence;
 }
@@ -143,46 +158,6 @@ impl<'a> HelperContextWrapper for HelperContext<'a> {
         self.lock.flush()
     }
 
-    fn buffer_map_directly(
-        &self,
-        res: &PipeResource,
-        offset: i32,
-        size: i32,
-        rw: RWFlags,
-    ) -> Option<PipeTransfer> {
-        self.lock.buffer_map_directly(res, offset, size, rw)
-    }
-
-    fn buffer_map_coherent(
-        &self,
-        res: &PipeResource,
-        offset: i32,
-        size: i32,
-        rw: RWFlags,
-    ) -> Option<PipeTransfer> {
-        self.lock
-            .buffer_map(res, offset, size, rw, ResourceMapType::Coherent)
-    }
-
-    fn texture_map_directly(
-        &self,
-        res: &PipeResource,
-        bx: &pipe_box,
-        rw: RWFlags,
-    ) -> Option<PipeTransfer> {
-        self.lock.texture_map_directly(res, bx, rw)
-    }
-
-    fn texture_map_coherent(
-        &self,
-        res: &PipeResource,
-        bx: &pipe_box,
-        rw: RWFlags,
-    ) -> Option<PipeTransfer> {
-        self.lock
-            .texture_map(res, bx, rw, ResourceMapType::Coherent)
-    }
-
     fn create_compute_state(&self, nir: &NirShader, static_local_mem: u32) -> *mut c_void {
         self.lock.create_compute_state(nir, static_local_mem)
     }
@@ -199,10 +174,6 @@ impl<'a> HelperContextWrapper for HelperContext<'a> {
         self.lock.compute_state_subgroup_size(state, block)
     }
 
-    fn unmap(&self, tx: PipeTransfer) {
-        tx.with_ctx(&self.lock);
-    }
-
     fn is_create_fence_fd_supported(&self) -> bool {
         self.lock.is_create_fence_fd_supported()
     }
@@ -230,6 +201,7 @@ impl Device {
         }
 
         let mut d = Self {
+            caps: DeviceCaps::new(&screen),
             base: CLObjectBase::new(RusticlTypes::Device),
             helper_ctx: Mutex::new(helper_ctx),
             screen: screen,
@@ -238,7 +210,6 @@ impl Device {
             clc_versions: Vec::new(),
             custom: false,
             embedded: false,
-            has_timestamp: false,
             extension_string: String::from(""),
             extensions: Vec::new(),
             spirv_extensions: Vec::new(),
@@ -247,17 +218,13 @@ impl Device {
             lib_clc: lib_clc?,
         };
 
-        d.fill_format_tables();
-
         // check if we are embedded or full profile first
         d.embedded = d.check_embedded_profile();
 
         // check if we have to report it as a custom device
         d.custom = d.check_custom();
 
-        let cap_timestamp = d.screen.param(pipe_cap::PIPE_CAP_QUERY_TIMESTAMP);
-        let cap_timestamp_res = d.timer_resolution();
-        d.has_timestamp = cap_timestamp != 0 && cap_timestamp_res > 0;
+        d.fill_format_tables();
 
         // query supported extensions
         d.fill_extensions();
@@ -275,6 +242,11 @@ impl Device {
     }
 
     fn fill_format_tables(&mut self) {
+        // no need to do this if we don't support images
+        if !self.caps.has_images {
+            return;
+        }
+
         for f in FORMATS {
             let mut fs = HashMap::new();
             for t in CL_IMAGE_TYPES {
@@ -305,9 +277,7 @@ impl Device {
                         PIPE_BIND_SHADER_IMAGE,
                     )
                 {
-                    flags |= CL_MEM_WRITE_ONLY;
-                    // TODO: enable once we support it
-                    // flags |= CL_MEM_KERNEL_READ_AND_WRITE;
+                    flags |= CL_MEM_WRITE_ONLY | CL_MEM_KERNEL_READ_AND_WRITE;
                 }
 
                 // TODO: cl_khr_srgb_image_writes
@@ -325,6 +295,57 @@ impl Device {
             }
             self.formats.insert(f.cl_image_format, fs);
         }
+
+        // now enable some caps based on advertized formats
+        self.caps.has_3d_image_writes = !FORMATS
+            .iter()
+            .filter(|f| {
+                if self.embedded {
+                    f.req_for_embeded_read_or_write
+                } else {
+                    f.req_for_full_read_or_write
+                }
+            })
+            .map(|f| self.formats[&f.cl_image_format][&CL_MEM_OBJECT_IMAGE3D])
+            .any(|f| f & cl_mem_flags::from(CL_MEM_WRITE_ONLY) == 0);
+
+        // if we can't advertize 3d image write ext, we have to disable them all
+        if !self.caps.has_3d_image_writes {
+            for f in &mut self.formats.values_mut() {
+                *f.get_mut(&CL_MEM_OBJECT_IMAGE3D).unwrap() &= !cl_mem_flags::from(
+                    CL_MEM_WRITE_ONLY | CL_MEM_READ_WRITE | CL_MEM_KERNEL_READ_AND_WRITE,
+                );
+            }
+        }
+
+        // we require formatted loads
+        if self.screen.param(pipe_cap::PIPE_CAP_IMAGE_LOAD_FORMATTED) != 0 {
+            // "For embedded profiles devices that support reading from and writing to the same
+            // image object from the same kernel instance (see CL_DEVICE_MAX_READ_WRITE_IMAGE_ARGS)
+            // there is no required minimum list of supported image formats."
+            self.caps.has_rw_images = if self.embedded {
+                FORMATS
+                    .iter()
+                    .flat_map(|f| self.formats[&f.cl_image_format].values())
+                    .any(|f| f & cl_mem_flags::from(CL_MEM_KERNEL_READ_AND_WRITE) != 0)
+            } else {
+                !FORMATS
+                    .iter()
+                    .filter(|f| f.req_for_full_read_and_write)
+                    .flat_map(|f| &self.formats[&f.cl_image_format])
+                    // maybe? things being all optional is kinda a mess
+                    .filter(|(target, _)| **target != CL_MEM_OBJECT_IMAGE3D)
+                    .any(|(_, mask)| mask & cl_mem_flags::from(CL_MEM_KERNEL_READ_AND_WRITE) == 0)
+            }
+        }
+
+        // if we can't advertize read_write images, disable them all
+        if !self.caps.has_rw_images {
+            self.formats
+                .values_mut()
+                .flat_map(|f| f.values_mut())
+                .for_each(|f| *f &= !cl_mem_flags::from(CL_MEM_KERNEL_READ_AND_WRITE));
+        }
     }
 
     fn check_valid(screen: &PipeScreen) -> bool {
@@ -419,15 +440,15 @@ impl Device {
     }
 
     fn check_embedded_profile(&self) -> bool {
-        if self.image_supported() {
+        if self.caps.has_images {
             // The minimum value is 16 if CL_DEVICE_IMAGE_SUPPORT is CL_TRUE
             if self.max_samplers() < 16 ||
             // The minimum value is 128 if CL_DEVICE_IMAGE_SUPPORT is CL_TRUE
-            self.image_read_count() < 128 ||
+            self.caps.max_read_images < 128 ||
             // The minimum value is 64 if CL_DEVICE_IMAGE_SUPPORT is CL_TRUE
-            self.image_write_count() < 64 ||
+            self.caps.max_write_images < 64 ||
             // The minimum value is 16384 if CL_DEVICE_IMAGE_SUPPORT is CL_TRUE
-            self.image_2d_size() < 16384 ||
+            self.caps.image_2d_size < 16384 ||
             // The minimum value is 2048 if CL_DEVICE_IMAGE_SUPPORT is CL_TRUE
             self.image_array_size() < 2048 ||
             // The minimum value is 65536 if CL_DEVICE_IMAGE_SUPPORT is CL_TRUE
@@ -470,7 +491,7 @@ impl Device {
         let mut res = CLVersion::Cl3_0;
 
         if self.embedded {
-            if self.image_supported() {
+            if self.caps.has_images {
                 let supports_array_writes = !FORMATS
                     .iter()
                     .filter(|f| f.req_for_embeded_read_or_write)
@@ -484,7 +505,7 @@ impl Device {
         }
 
         // TODO: check image 1D, 1Dbuffer, 1Darray and 2Darray support explicitly
-        if self.image_supported() {
+        if self.caps.has_images {
             // The minimum value is 256 if CL_DEVICE_IMAGE_SUPPORT is CL_TRUE
             if self.image_array_size() < 256 ||
             // The minimum value is 2048 if CL_DEVICE_IMAGE_SUPPORT is CL_TRUE
@@ -604,18 +625,18 @@ impl Device {
             add_feat(1, 0, 0, "__opencl_c_int64");
         }
 
-        if self.image_supported() {
+        if self.caps.has_images {
             add_feat(1, 0, 0, "__opencl_c_images");
 
             if self.image2d_from_buffer_supported() {
                 add_ext(1, 0, 0, "cl_khr_image2d_from_buffer");
             }
 
-            if self.image_read_write_supported() {
+            if self.caps.has_rw_images {
                 add_feat(1, 0, 0, "__opencl_c_read_write_images");
             }
 
-            if self.image_3d_write_supported() {
+            if self.caps.has_3d_image_writes {
                 add_ext(1, 0, 0, "cl_khr_3d_image_writes");
                 add_feat(1, 0, 0, "__opencl_c_3d_image_writes");
             }
@@ -798,83 +819,63 @@ impl Device {
         }
     }
 
-    pub fn image_2d_size(&self) -> usize {
-        self.screen.param(pipe_cap::PIPE_CAP_MAX_TEXTURE_2D_SIZE) as usize
-    }
-
     pub fn image_3d_size(&self) -> usize {
-        1 << (self.screen.param(pipe_cap::PIPE_CAP_MAX_TEXTURE_3D_LEVELS) - 1)
+        if self.caps.has_images {
+            1 << (self.screen.param(pipe_cap::PIPE_CAP_MAX_TEXTURE_3D_LEVELS) - 1)
+        } else {
+            0
+        }
     }
 
     pub fn image_3d_supported(&self) -> bool {
-        self.screen.param(pipe_cap::PIPE_CAP_MAX_TEXTURE_3D_LEVELS) != 0
+        self.caps.has_images && self.screen.param(pipe_cap::PIPE_CAP_MAX_TEXTURE_3D_LEVELS) != 0
     }
 
     pub fn image_array_size(&self) -> usize {
-        self.screen
-            .param(pipe_cap::PIPE_CAP_MAX_TEXTURE_ARRAY_LAYERS) as usize
+        if self.caps.has_images {
+            self.screen
+                .param(pipe_cap::PIPE_CAP_MAX_TEXTURE_ARRAY_LAYERS) as usize
+        } else {
+            0
+        }
     }
 
     pub fn image_pitch_alignment(&self) -> cl_uint {
-        self.screen
-            .param(pipe_cap::PIPE_CAP_LINEAR_IMAGE_PITCH_ALIGNMENT) as u32
+        if self.caps.has_images {
+            self.screen
+                .param(pipe_cap::PIPE_CAP_LINEAR_IMAGE_PITCH_ALIGNMENT) as u32
+        } else {
+            0
+        }
     }
 
     pub fn image_base_address_alignment(&self) -> cl_uint {
-        self.screen
-            .param(pipe_cap::PIPE_CAP_LINEAR_IMAGE_BASE_ADDRESS_ALIGNMENT) as u32
+        if self.caps.has_images {
+            self.screen
+                .param(pipe_cap::PIPE_CAP_LINEAR_IMAGE_BASE_ADDRESS_ALIGNMENT) as u32
+        } else {
+            0
+        }
     }
 
     pub fn image_buffer_size(&self) -> usize {
-        min(
-            // the CTS requires it to not exceed `CL_MAX_MEM_ALLOC_SIZE`
-            self.max_mem_alloc(),
-            self.screen
-                .param(pipe_cap::PIPE_CAP_MAX_TEXEL_BUFFER_ELEMENTS_UINT) as cl_ulong,
-        ) as usize
-    }
-
-    pub fn image_read_count(&self) -> cl_uint {
-        self.shader_param(pipe_shader_cap::PIPE_SHADER_CAP_MAX_SAMPLER_VIEWS) as cl_uint
+        if self.caps.has_images {
+            min(
+                // the CTS requires it to not exceed `CL_MAX_MEM_ALLOC_SIZE`
+                self.max_mem_alloc(),
+                self.screen
+                    .param(pipe_cap::PIPE_CAP_MAX_TEXEL_BUFFER_ELEMENTS_UINT)
+                    as cl_ulong,
+            ) as usize
+        } else {
+            0
+        }
     }
 
     pub fn image2d_from_buffer_supported(&self) -> bool {
         self.image_pitch_alignment() != 0 && self.image_base_address_alignment() != 0
     }
 
-    pub fn image_supported(&self) -> bool {
-        // TODO check CL_DEVICE_IMAGE_SUPPORT reqs
-        self.shader_param(pipe_shader_cap::PIPE_SHADER_CAP_MAX_SHADER_IMAGES) != 0 &&
-      // The minimum value is 8 if CL_DEVICE_IMAGE_SUPPORT is CL_TRUE
-      self.image_read_count() >= 8 &&
-      // The minimum value is 8 if CL_DEVICE_IMAGE_SUPPORT is CL_TRUE
-      self.image_write_count() >= 8 &&
-      // The minimum value is 2048 if CL_DEVICE_IMAGE_SUPPORT is CL_TRUE
-      self.image_2d_size() >= 2048
-    }
-
-    pub fn image_read_write_supported(&self) -> bool {
-        !FORMATS
-            .iter()
-            .filter(|f| f.req_for_full_read_and_write)
-            .map(|f| self.formats.get(&f.cl_image_format).unwrap())
-            .map(|f| f.get(&CL_MEM_OBJECT_IMAGE3D).unwrap())
-            .any(|f| *f & cl_mem_flags::from(CL_MEM_KERNEL_READ_AND_WRITE) == 0)
-    }
-
-    pub fn image_3d_write_supported(&self) -> bool {
-        !FORMATS
-            .iter()
-            .filter(|f| f.req_for_full_read_or_write)
-            .map(|f| self.formats.get(&f.cl_image_format).unwrap())
-            .map(|f| f.get(&CL_MEM_OBJECT_IMAGE3D).unwrap())
-            .any(|f| *f & cl_mem_flags::from(CL_MEM_WRITE_ONLY) == 0)
-    }
-
-    pub fn image_write_count(&self) -> cl_uint {
-        self.shader_param(pipe_shader_cap::PIPE_SHADER_CAP_MAX_SHADER_IMAGES) as cl_uint
-    }
-
     pub fn little_endian(&self) -> bool {
         let endianness = self.screen.param(pipe_cap::PIPE_CAP_ENDIANNESS);
         endianness == (pipe_endian::PIPE_ENDIAN_LITTLE as i32)
@@ -892,6 +893,16 @@ impl Device {
         v.into_iter().map(|v| v as usize).collect()
     }
 
+    pub fn max_grid_size(&self) -> Vec<u64> {
+        let v: Vec<u64> = self
+            .screen
+            .compute_param(pipe_compute_cap::PIPE_COMPUTE_CAP_MAX_GRID_SIZE);
+
+        v.into_iter()
+            .map(|a| min(a, Platform::dbg().max_grid_size))
+            .collect()
+    }
+
     pub fn max_clock_freq(&self) -> cl_uint {
         self.screen
             .compute_param(pipe_compute_cap::PIPE_COMPUTE_CAP_MAX_CLOCK_FREQUENCY)
@@ -993,10 +1004,6 @@ impl Device {
         self.screen.param(pipe_cap::PIPE_CAP_SYSTEM_SVM) == 1
     }
 
-    pub fn timer_resolution(&self) -> usize {
-        self.screen.param(pipe_cap::PIPE_CAP_TIMER_RESOLUTION) as usize
-    }
-
     pub fn unified_memory(&self) -> bool {
         self.screen.param(pipe_cap::PIPE_CAP_UMA) == 1
     }
@@ -1039,9 +1046,9 @@ impl Device {
             fp16: self.fp16_supported(),
             fp64: self.fp64_supported(),
             int64: self.int64_supported(),
-            images: self.image_supported(),
-            images_read_write: self.image_read_write_supported(),
-            images_write_3d: self.image_3d_write_supported(),
+            images: self.caps.has_images,
+            images_read_write: self.caps.has_rw_images,
+            images_write_3d: self.caps.has_3d_image_writes,
             integer_dot_product: true,
             subgroups: subgroups_supported,
             subgroups_shuffle: subgroups_supported,
