@@ -12,6 +12,8 @@
 #include "nvk_instance.h"
 #include "nvk_shader.h"
 #include "nvk_wsi.h"
+#include "nvkmd/nvkmd.h"
+#include "nvkmd/nouveau/nvkmd_nouveau.h"
 #include "git_sha1.h"
 #include "util/disk_cache.h"
 #include "util/mesa-sha1.h"
@@ -21,10 +23,7 @@
 #include "vk_shader_module.h"
 #include "vulkan/wsi/wsi_common.h"
 
-#include <fcntl.h>
-#include <sys/stat.h>
 #include <sys/sysmacros.h>
-#include <xf86drm.h>
 
 #include "cl90c0.h"
 #include "cl91c0.h"
@@ -175,11 +174,13 @@ nvk_get_device_extensions(const struct nvk_instance *instance,
       .EXT_buffer_device_address = true,
       .EXT_calibrated_timestamps = true,
       .EXT_conditional_rendering = true,
+      .EXT_conservative_rasterization = info->cls_eng3d >= MAXWELL_B,
       .EXT_color_write_enable = true,
       .EXT_custom_border_color = true,
       .EXT_depth_bias_control = true,
       .EXT_depth_clip_control = true,
       .EXT_depth_clip_enable = true,
+      .EXT_depth_range_unrestricted = info->cls_eng3d >= VOLTA_A,
       .EXT_descriptor_indexing = true,
 #ifdef VK_USE_PLATFORM_DISPLAY_KHR
       .EXT_display_control = true,
@@ -209,6 +210,7 @@ nvk_get_device_extensions(const struct nvk_instance *instance,
       .EXT_pci_bus_info = info->type == NV_DEVICE_TYPE_DIS,
       .EXT_pipeline_creation_cache_control = true,
       .EXT_pipeline_creation_feedback = true,
+      .EXT_pipeline_robustness = true,
       .EXT_physical_device_drm = true,
       .EXT_primitive_topology_list_restart = true,
       .EXT_private_data = true,
@@ -225,6 +227,7 @@ nvk_get_device_extensions(const struct nvk_instance *instance,
       .EXT_shader_demote_to_helper_invocation = true,
       .EXT_shader_module_identifier = true,
       .EXT_shader_object = true,
+      .EXT_shader_replicated_composites = true,
       .EXT_shader_subgroup_ballot = true,
       .EXT_shader_subgroup_vote = true,
       .EXT_shader_viewport_index_layer = info->cls_eng3d >= MAXWELL_B,
@@ -299,13 +302,13 @@ nvk_get_device_features(const struct nv_device_info *info,
       .shaderResourceResidency = info->cls_eng3d >= VOLTA_A,
       .shaderResourceMinLod = info->cls_eng3d >= VOLTA_A,
       .sparseBinding = true,
-      .sparseResidency2Samples = info->cls_eng3d >= MAXWELL_A,
-      .sparseResidency4Samples = info->cls_eng3d >= MAXWELL_A,
-      .sparseResidency8Samples = info->cls_eng3d >= MAXWELL_A,
-      .sparseResidencyAliased = info->cls_eng3d >= MAXWELL_A,
-      .sparseResidencyBuffer = info->cls_eng3d >= MAXWELL_A,
-      .sparseResidencyImage2D = info->cls_eng3d >= MAXWELL_A,
-      .sparseResidencyImage3D = info->cls_eng3d >= MAXWELL_A,
+      .sparseResidency2Samples = info->cls_eng3d >= MAXWELL_B,
+      .sparseResidency4Samples = info->cls_eng3d >= MAXWELL_B,
+      .sparseResidency8Samples = info->cls_eng3d >= MAXWELL_B,
+      .sparseResidencyAliased = info->cls_eng3d >= MAXWELL_B,
+      .sparseResidencyBuffer = info->cls_eng3d >= MAXWELL_B,
+      .sparseResidencyImage2D = info->cls_eng3d >= MAXWELL_B,
+      .sparseResidencyImage3D = info->cls_eng3d >= MAXWELL_B,
       .variableMultisampleRate = true,
       .inheritedQueries = true,
 
@@ -441,8 +444,8 @@ nvk_get_device_features(const struct nv_device_info *info,
       /* VK_KHR_workgroup_memory_explicit_layout */
       .workgroupMemoryExplicitLayout = true,
       .workgroupMemoryExplicitLayoutScalarBlockLayout = true,
-      .workgroupMemoryExplicitLayout8BitAccess = false,
-      .workgroupMemoryExplicitLayout16BitAccess = false,
+      .workgroupMemoryExplicitLayout8BitAccess = nvk_use_nak(info),
+      .workgroupMemoryExplicitLayout16BitAccess = nvk_use_nak(info),
 
       /* VK_EXT_4444_formats */
       .formatA4R4G4B4 = true,
@@ -562,6 +565,9 @@ nvk_get_device_features(const struct nv_device_info *info,
       /* VK_EXT_non_seamless_cube_map */
       .nonSeamlessCubeMap = true,
 
+      /* VK_EXT_pipeline_robustness */
+      .pipelineRobustness = true,
+
       /* VK_EXT_primitive_topology_list_restart */
       .primitiveTopologyListRestart = true,
       .primitiveTopologyPatchListRestart = true,
@@ -592,6 +598,9 @@ nvk_get_device_features(const struct nv_device_info *info,
       /* VK_EXT_shader_object */
       .shaderObject = true,
 
+      /* VK_EXT_shader_replicated_composites */
+      .shaderReplicatedComposites = true,
+
       /* VK_KHR_shader_subgroup_uniform_control_flow */
       .shaderSubgroupUniformControlFlow = nvk_use_nak(info),
 
@@ -616,12 +625,6 @@ nvk_get_device_features(const struct nv_device_info *info,
    };
 }
 
-uint32_t
-nvk_min_cbuf_alignment(const struct nv_device_info *info)
-{
-   return info->cls_eng3d >= TURING_A ? 64 : 256;
-}
-
 static void
 nvk_get_device_properties(const struct nvk_instance *instance,
                           const struct nv_device_info *info,
@@ -633,6 +636,8 @@ nvk_get_device_properties(const struct nvk_instance *instance,
                                                VK_SAMPLE_COUNT_4_BIT |
                                                VK_SAMPLE_COUNT_8_BIT;
 
+   assert(sample_counts <= (NVK_MAX_SAMPLES << 1) - 1);
+
    uint64_t os_page_size = 4096;
    os_get_page_size(&os_page_size);
 
@@ -866,6 +871,17 @@ nvk_get_device_properties(const struct nvk_instance *instance,
       /* VK_KHR_push_descriptor */
       .maxPushDescriptors = NVK_MAX_PUSH_DESCRIPTORS,
 
+      /* VK_EXT_conservative_rasterization */
+      .primitiveOverestimationSize = info->cls_eng3d >= VOLTA_A ? 1.0f / 512.0f : 0.0,
+      .maxExtraPrimitiveOverestimationSize = 0.75,
+      .extraPrimitiveOverestimationSizeGranularity = 0.25,
+      .primitiveUnderestimation = info->cls_eng3d >= VOLTA_A,
+      .conservativePointAndLineRasterization = true,
+      .degenerateLinesRasterized = info->cls_eng3d >= VOLTA_A,
+      .degenerateTrianglesRasterized = info->cls_eng3d >= PASCAL_A,
+      .fullyCoveredFragmentShaderInputVariable = false,
+      .conservativeRasterizationPostDepthCoverage = true,
+
       /* VK_EXT_custom_border_color */
       .maxCustomBorderColorSamplers = 4000,
 
@@ -907,6 +923,16 @@ nvk_get_device_properties(const struct nvk_instance *instance,
       .pciDevice   = info->pci.dev,
       .pciFunction = info->pci.func,
 
+      /* VK_EXT_pipeline_robustness */
+      .defaultRobustnessStorageBuffers =
+         VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_DISABLED_EXT,
+      .defaultRobustnessUniformBuffers =
+         VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_DISABLED_EXT,
+      .defaultRobustnessVertexInputs =
+         VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS_2_EXT,
+      .defaultRobustnessImages =
+         VK_PIPELINE_ROBUSTNESS_IMAGE_BEHAVIOR_ROBUST_IMAGE_ACCESS_2_EXT,
+
       /* VK_EXT_physical_device_drm gets populated later */
 
       /* VK_EXT_provoking_vertex */
@@ -968,16 +994,7 @@ nvk_get_device_properties(const struct nvk_instance *instance,
             vk_shaderModuleIdentifierAlgorithmUUID,
             sizeof(properties->shaderModuleIdentifierAlgorithmUUID));
 
-   const struct {
-      uint16_t vendor_id;
-      uint16_t device_id;
-      uint8_t pad[12];
-   } dev_uuid = {
-      .vendor_id = NVIDIA_VENDOR_ID,
-      .device_id = info->device_id,
-   };
-   STATIC_ASSERT(sizeof(dev_uuid) == VK_UUID_SIZE);
-   memcpy(properties->deviceUUID, &dev_uuid, VK_UUID_SIZE);
+   nv_device_uuid(info, properties->deviceUUID, VK_UUID_SIZE, true);
    STATIC_ASSERT(sizeof(instance->driver_build_sha) >= VK_UUID_SIZE);
    memcpy(properties->driverUUID, instance->driver_build_sha, VK_UUID_SIZE);
 
@@ -1061,90 +1078,37 @@ nvk_get_sysmem_heap_available(struct nvk_physical_device *pdev)
 static uint64_t
 nvk_get_vram_heap_available(struct nvk_physical_device *pdev)
 {
-   const uint64_t used = nouveau_ws_device_vram_used(pdev->ws_dev);
+   const uint64_t used = nvkmd_pdev_get_vram_used(pdev->nvkmd);
    if (used > pdev->info.vram_size_B)
       return 0;
 
    return pdev->info.vram_size_B - used;
 }
 
-static bool
-drm_device_is_nouveau(const char *path)
-{
-   int fd = open(path, O_RDWR | O_CLOEXEC);
-   if (fd < 0)
-      return false;
-
-   drmVersionPtr ver = drmGetVersion(fd);
-   if (!ver) {
-      close(fd);
-      return false;
-   }
-
-   const bool is_nouveau = !strncmp("nouveau", ver->name, ver->name_len);
-
-   drmFreeVersion(ver);
-   close(fd);
-
-   return is_nouveau;
-}
-
 VkResult
 nvk_create_drm_physical_device(struct vk_instance *_instance,
-                               drmDevicePtr drm_device,
+                               struct _drmDevice *drm_device,
                                struct vk_physical_device **pdev_out)
 {
    struct nvk_instance *instance = (struct nvk_instance *)_instance;
    VkResult result;
-   int master_fd = -1;
 
-   if (!(drm_device->available_nodes & (1 << DRM_NODE_RENDER)))
-      return VK_ERROR_INCOMPATIBLE_DRIVER;
-
-   switch (drm_device->bustype) {
-   case DRM_BUS_PCI:
-      if (drm_device->deviceinfo.pci->vendor_id != NVIDIA_VENDOR_ID)
-         return VK_ERROR_INCOMPATIBLE_DRIVER;
-      break;
-
-   case DRM_BUS_PLATFORM: {
-      const char *compat_prefix = "nvidia,";
-      bool found = false;
-      for (int i = 0; drm_device->deviceinfo.platform->compatible[i] != NULL; i++) {
-         if (strncmp(drm_device->deviceinfo.platform->compatible[0], compat_prefix, strlen(compat_prefix)) == 0) {
-            found = true;
-            break;
-         }
-      }
-      if (!found)
-         return VK_ERROR_INCOMPATIBLE_DRIVER;
-      break;
-   }
-
-   default:
-      return VK_ERROR_INCOMPATIBLE_DRIVER;
-   }
-
-   if (!drm_device_is_nouveau(drm_device->nodes[DRM_NODE_RENDER]))
-      return VK_ERROR_INCOMPATIBLE_DRIVER;
-
-   struct nouveau_ws_device *ws_dev = nouveau_ws_device_new(drm_device);
-   if (!ws_dev)
-      return vk_error(instance, VK_ERROR_INCOMPATIBLE_DRIVER);
-
-   const struct nv_device_info info = ws_dev->info;
-   const struct vk_sync_type syncobj_sync_type =
-      vk_drm_syncobj_get_type(ws_dev->fd);
+   struct nvkmd_pdev *nvkmd;
+   result = nvkmd_try_create_pdev_for_drm(drm_device, &instance->vk.base,
+                                          instance->debug_flags, &nvkmd);
+   if (result != VK_SUCCESS)
+      return result;
 
    /* We don't support anything pre-Kepler */
-   if (info.cls_eng3d < KEPLER_A) {
+   if (nvkmd->dev_info.cls_eng3d < KEPLER_A) {
       result = VK_ERROR_INCOMPATIBLE_DRIVER;
-      goto fail_ws_dev;
+      goto fail_nvkmd;
    }
 
    bool conformant =
-      info.type == NV_DEVICE_TYPE_DIS &&
-      info.cls_eng3d >= TURING_A && info.cls_eng3d <= ADA_A;
+      nvkmd->dev_info.type == NV_DEVICE_TYPE_DIS &&
+      nvkmd->dev_info.cls_eng3d >= TURING_A &&
+      nvkmd->dev_info.cls_eng3d <= ADA_A;
 
    if (!conformant &&
        !debug_get_bool_option("NVK_I_WANT_A_BROKEN_VULKAN_DRIVER", false)) {
@@ -1155,26 +1119,11 @@ nvk_create_drm_physical_device(struct vk_instance *_instance,
                          "WARNING: NVK is not well-tested on %s, pass "
                          "NVK_I_WANT_A_BROKEN_VULKAN_DRIVER=1 "
                          "if you know what you're doing.",
-                         info.device_name);
+                         nvkmd->dev_info.device_name);
 #endif
-      goto fail_ws_dev;
+      goto fail_nvkmd;
    }
 
-   if (!ws_dev->has_vm_bind) {
-      result = vk_errorf(instance, VK_ERROR_INCOMPATIBLE_DRIVER,
-                         "NVK Requires a Linux kernel version 6.6 or later");
-      goto fail_ws_dev;
-   }
-
-   struct stat st;
-   if (stat(drm_device->nodes[DRM_NODE_RENDER], &st)) {
-      result = vk_errorf(instance, VK_ERROR_INITIALIZATION_FAILED,
-                         "fstat() failed on %s: %m",
-                         drm_device->nodes[DRM_NODE_RENDER]);
-      goto fail_ws_dev;
-   }
-   const dev_t render_dev = st.st_rdev;
-
    if (!conformant)
       vk_warn_non_conformant_implementation("NVK");
 
@@ -1184,7 +1133,7 @@ nvk_create_drm_physical_device(struct vk_instance *_instance,
 
    if (pdev == NULL) {
       result = vk_error(instance, VK_ERROR_OUT_OF_HOST_MEMORY);
-      goto fail_ws_dev;
+      goto fail_nvkmd;
    }
 
    struct vk_physical_device_dispatch_table dispatch_table;
@@ -1193,32 +1142,29 @@ nvk_create_drm_physical_device(struct vk_instance *_instance,
    vk_physical_device_dispatch_table_from_entrypoints(
       &dispatch_table, &wsi_physical_device_entrypoints, false);
 
-   const bool has_tiled_bos = nouveau_ws_device_has_tiled_bo(ws_dev);
    struct vk_device_extension_table supported_extensions;
-   nvk_get_device_extensions(instance, &info, has_tiled_bos,
+   nvk_get_device_extensions(instance, &nvkmd->dev_info,
+                             nvkmd->kmd_info.has_alloc_tiled,
                              &supported_extensions);
 
    struct vk_features supported_features;
-   nvk_get_device_features(&info, &supported_extensions, &supported_features);
+   nvk_get_device_features(&nvkmd->dev_info, &supported_extensions,
+                           &supported_features);
 
    struct vk_properties properties;
-   nvk_get_device_properties(instance, &info, conformant, &properties);
+   nvk_get_device_properties(instance, &nvkmd->dev_info, conformant,
+                             &properties);
 
-   properties.drmHasRender = true;
-   properties.drmRenderMajor = major(render_dev);
-   properties.drmRenderMinor = minor(render_dev);
+   if (nvkmd->drm.render_dev) {
+      properties.drmHasRender = true;
+      properties.drmRenderMajor = major(nvkmd->drm.render_dev);
+      properties.drmRenderMinor = minor(nvkmd->drm.render_dev);
+   }
 
-   /* DRM primary is optional */
-   if ((drm_device->available_nodes & (1 << DRM_NODE_PRIMARY)) &&
-       !stat(drm_device->nodes[DRM_NODE_PRIMARY], &st)) {
-      assert(st.st_rdev != 0);
+   if (nvkmd->drm.primary_dev) {
       properties.drmHasPrimary = true;
-      properties.drmPrimaryMajor = major(st.st_rdev);
-      properties.drmPrimaryMinor = minor(st.st_rdev);
-
-      /* TODO: Test if the FD is usable? */
-      if (instance->vk.enabled_extensions.KHR_display)
-         master_fd = open(drm_device->nodes[DRM_NODE_PRIMARY], O_RDWR | O_CLOEXEC);
+      properties.drmPrimaryMajor = major(nvkmd->drm.primary_dev);
+      properties.drmPrimaryMinor = minor(nvkmd->drm.primary_dev);
    }
 
    result = vk_physical_device_init(&pdev->vk, &instance->vk,
@@ -1227,13 +1173,11 @@ nvk_create_drm_physical_device(struct vk_instance *_instance,
                                     &properties,
                                     &dispatch_table);
    if (result != VK_SUCCESS)
-      goto fail_master_fd;
+      goto fail_alloc;
 
-   pdev->info = info;
-   pdev->debug_flags = ws_dev->debug_flags;
-   pdev->render_dev = render_dev;
-   pdev->master_fd = master_fd;
-   pdev->ws_dev = ws_dev;
+   pdev->nvkmd = nvkmd;
+   pdev->info = nvkmd->dev_info;
+   pdev->debug_flags = instance->debug_flags;
 
    pdev->nak = nak_compiler_create(&pdev->info);
    if (pdev->nak == NULL) {
@@ -1268,7 +1212,7 @@ nvk_create_drm_physical_device(struct vk_instance *_instance,
       }
 
       /* Only set available if we have the ioctl. */
-      if (nouveau_ws_device_vram_used(ws_dev) > 0)
+      if (nvkmd->kmd_info.has_get_vram_used)
          pdev->mem_heaps[vram_heap_idx].available = nvk_get_vram_heap_available;
 
       pdev->mem_types[pdev->mem_type_count++] = (VkMemoryType) {
@@ -1316,12 +1260,7 @@ nvk_create_drm_physical_device(struct vk_instance *_instance,
    };
    assert(pdev->queue_family_count <= ARRAY_SIZE(pdev->queue_families));
 
-   unsigned st_idx = 0;
-   pdev->syncobj_sync_type = syncobj_sync_type;
-   pdev->sync_types[st_idx++] = &pdev->syncobj_sync_type;
-   pdev->sync_types[st_idx++] = NULL;
-   assert(st_idx <= ARRAY_SIZE(pdev->sync_types));
-   pdev->vk.supported_sync_types = pdev->sync_types;
+   pdev->vk.supported_sync_types = nvkmd->sync_types;
 
    result = nvk_init_wsi(pdev);
    if (result != VK_SUCCESS)
@@ -1336,12 +1275,10 @@ fail_disk_cache:
    nak_compiler_destroy(pdev->nak);
 fail_init:
    vk_physical_device_finish(&pdev->vk);
-fail_master_fd:
-   if (master_fd >= 0)
-      close(master_fd);
+fail_alloc:
    vk_free(&instance->vk.alloc, pdev);
-fail_ws_dev:
-   nouveau_ws_device_destroy(ws_dev);
+fail_nvkmd:
+   nvkmd_pdev_destroy(nvkmd);
    return result;
 }
 
@@ -1354,9 +1291,7 @@ nvk_physical_device_destroy(struct vk_physical_device *vk_pdev)
    nvk_finish_wsi(pdev);
    nvk_physical_device_free_disk_cache(pdev);
    nak_compiler_destroy(pdev->nak);
-   if (pdev->master_fd >= 0)
-      close(pdev->master_fd);
-   nouveau_ws_device_destroy(pdev->ws_dev);
+   nvkmd_pdev_destroy(pdev->nvkmd);
    vk_physical_device_finish(&pdev->vk);
    vk_free(&pdev->vk.instance->alloc, pdev);
 }
@@ -1446,7 +1381,7 @@ nvk_GetPhysicalDeviceMemoryProperties2(
          break;
       }
       default:
-         nvk_debug_ignored_stype(ext->sType);
+         vk_debug_ignored_stype(ext->sType);
          break;
       }
    }
